package Local::LongList;

use strict;
use warnings;

use Exporter qw( import );

sub alpha   { }
sub bravo   { }
sub charlie { }
sub delta   { }
sub echo    { }
sub foxtrot { }
sub golf    { }
sub hotel   { }
sub india   { }
sub juliet  { }

our @EXPORT_OK = qw(
    alpha
    bravo
    charlie
    delta
    echo
    foxtrot
    golf
    hotel
    india
    juliet
);

1;
