use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007006
use Test::Spelling 0.17;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
1197072
198982749
49699333
Alders
Alders'
Annotations
App
CLI
Choroba
Config
Document
ExportInspector
Include
Ishigaki
James
Johnson
José
Kenichi
Logger
Manuel
Mendoza
Nelo
Nicolas
Olaf
Oliver
Onyiah
PPI
Paul
Peter
Raspass
Rodríguez
Role
Sandbox
choroba
copilot
dependabot
dump
git
ishigaki
jose93rd
jraspass
lib
mendoza
nelo
olaf
paul
perlimports
pyrrhlin
sandboxed
script
